@@ -1,9 +1,10 @@
 import asyncio
+import functools
 import logging
 from abc import abstractmethod, abstractproperty
 from dataclasses import dataclass, field
 from timeit import default_timer
-from typing import Dict, List, Optional, Type, TypeVar, Union
+from typing import Any, Dict, List, Mapping, Optional, Pattern, Type, TypeVar, Union
 
 import faust
 from aioprometheus import Counter, Service, Summary
@@ -11,10 +12,15 @@ from aioprometheus.collectors import Histogram
 from betterproto import Message
 from confluent_kafka.admin import AdminClient, NewTopic
 from dacite import from_dict
+from dataclasses_avroschema import AvroModel
+from faust.models import Model
 from faust.serializers import Codec
 from faust.streams import Stream
-from faust.types import CodecArg
-from faust.types.models import ModelT
+from faust.types.serializers import CodecArg, SchemaT
+from faust.types.models import ModelT, ModelArg
+from faust.types.topics import Seconds, TopicT
+from schema_registry.client import SchemaRegistryClient
+from schema_registry.serializers.faust import FaustSerializer
 
 from delphai_utils.config import get_config
 from delphai_utils.patches import aiokafka_patch
@@ -158,7 +164,11 @@ class FaustApp:
         logger.info(f"shutting down {self.app._conf.id}")
 
     def __init__(
-        self, pipeline_name: str, broker: str, step: Union[str, None] = None
+        self,
+        pipeline_name: str,
+        broker: str,
+        step: Optional[str] = None,
+        schema_registry_url: Optional[str] = None,
     ) -> None:
         self.pipeline_name = pipeline_name
         app_id = self.pipeline_name if step is None else f"{self.pipeline_name}.{step}"
@@ -172,8 +182,12 @@ class FaustApp:
                     self.faust_config.update(faust_config.get(step, {}))
         except Exception:
             pass
-        self.app = faust.App(
-            id=app_id, broker=broker, web_enabled=False, **self.faust_config
+        self.app = _FaustApp(
+            id=app_id,
+            broker=broker,
+            web_enabled=False,
+            **self.faust_config,
+            schema_registry_url=schema_registry_url,
         )
         loop = asyncio.get_event_loop()
 
@@ -246,3 +260,82 @@ class FaustApp:
         agent.setup_metrics()
         await agent.after_attach()
         await new_agent.start()
+
+
+def get_avro_schema(cls):
+    @functools.wraps(cls, updated=[])
+    class WithAvroModel(dataclass(cls), AvroModel):
+        pass
+
+    return WithAvroModel.avro_schema()
+
+
+class _FaustApp(faust.App):
+    def __init__(self, *args, schema_registry_url=None, **kwargs):
+        self._delphai_schema_registry_client = (
+            SchemaRegistryClient(schema_registry_url) if schema_registry_url else None
+        )
+        super().__init__(*args, **kwargs)
+
+    def topic(
+        self,
+        *topics: str,
+        pattern: Union[str, Pattern] = None,
+        schema: SchemaT = None,
+        key_type: ModelArg = None,
+        value_type: ModelArg = None,
+        key_serializer: CodecArg = None,
+        value_serializer: CodecArg = None,
+        partitions: int = None,
+        retention: Seconds = None,
+        compacting: bool = None,
+        deleting: bool = None,
+        replicas: int = None,
+        acks: bool = True,
+        internal: bool = False,
+        config: Mapping[str, Any] = None,
+        maxsize: int = None,
+        allow_empty: bool = False,
+        has_prefix: bool = False,
+        loop: asyncio.AbstractEventLoop = None,
+    ) -> TopicT:
+        if self._delphai_schema_registry_client and (schema is None) and topics:
+            if (key_serializer is None) and key_type and issubclass(key_type, Model):
+                key_serializer = FaustSerializer(
+                    self._delphai_schema_registry_client,
+                    f"{topics[0]}-key",
+                    get_avro_schema(key_type),
+                )
+
+            if (
+                (value_serializer is None)
+                and value_type
+                and issubclass(value_type, Model)
+            ):
+                value_serializer = FaustSerializer(
+                    self._delphai_schema_registry_client,
+                    f"{topics[0]}-value",
+                    get_avro_schema(value_type),
+                )
+
+        return super().topic(
+            *topics,
+            pattern=pattern,
+            schema=schema,
+            key_type=key_type,
+            value_type=value_type,
+            key_serializer=key_serializer,
+            value_serializer=value_serializer,
+            partitions=partitions,
+            retention=retention,
+            compacting=compacting,
+            deleting=deleting,
+            replicas=replicas,
+            acks=acks,
+            internal=internal,
+            config=config,
+            maxsize=maxsize,
+            allow_empty=allow_empty,
+            has_prefix=has_prefix,
+            loop=loop,
+        )
