@@ -106,12 +106,13 @@ def start_server(
     start_http_server(metrics_port)
     logger.info(f"started metrics server on port {metrics_port}")
 
+    tasks = [loop.create_task(server.wait_for_termination())]
     if gateway:
-        start_gateway(server, grpc_port, http_port)
+        tasks.append(start_gateway(server, grpc_port, http_port))
 
-    loop.create_task(update_public_keys())
+    tasks.append(loop.create_task(update_public_keys()))
 
     try:
-        loop.run_until_complete(server.wait_for_termination())
+        loop.run_until_complete(asyncio.gather(*tasks))
     except KeyboardInterrupt:
         logger.info("stopped server (keyboard interrupt)")
